Class: Async::HTTP::Protocol::HTTP1::Request

Inherits:
Request
  • Object
show all
Defined in:
lib/async/http/protocol/http1/request.rb

Overview

An incoming HTTP/1 request parsed from the connection.

Constant Summary collapse

URI_PATTERN =
%r{\A(?<scheme>[^:/]+)://(?<authority>[^/]+)(?<path>.*)\z}
UPGRADE =
"upgrade"

Class Method Summary collapse

Instance Method Summary collapse

Methods inherited from Request

#inspect, #peer, #remote_address

Constructor Details

#initialize(connection, scheme, authority, method, path, version, headers, body) ⇒ Request

Initialize the request from the parsed components.



61
62
63
64
65
66
67
68
# File 'lib/async/http/protocol/http1/request.rb', line 61

def initialize(connection, scheme, authority, method, path, version, headers, body)
	@connection = connection
	
	# HTTP/1 requests with an upgrade header (which can contain zero or more values) are extracted into the protocol field of the request, and we expect a response to select one of those protocols with a status code of 101 Switching Protocols.
	protocol = headers.delete("upgrade")
	
	super(scheme, authority, method, path, version, headers, body, protocol, self.public_method(:write_interim_response))
end

Class Method Details

.read(connection) ⇒ Object

Read and parse the next request from the connection.



32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
# File 'lib/async/http/protocol/http1/request.rb', line 32

def self.read(connection)
	connection.read_request do |authority, method, target, version, headers, body|
		if method == ::Protocol::HTTP::Methods::CONNECT
			# We put the target into the authority field for CONNECT requests, as per HTTP/2 semantics.
			self.new(connection, nil, target, method, nil, version, headers, body)
		elsif valid_path?(target)
			# This is a valid request.
			self.new(connection, nil, authority, method, target, version, headers, body)
		elsif match = target.match(URI_PATTERN)
			# We map the incoming absolute URI target to the scheme, authority, and path fields of the request.
			self.new(connection, match[:scheme], match[:authority], method, match[:path], version, headers, body)
		else
			# This is an invalid request.
			raise ::Protocol::HTTP1::BadRequest.new("Invalid request target: #{target}")
		end
	end
end

.valid_path?(target) ⇒ Boolean

Check whether the given request target is a valid path.

Returns:

  • (Boolean)


17
18
19
20
21
22
23
24
25
# File 'lib/async/http/protocol/http1/request.rb', line 17

def self.valid_path?(target)
	if target.start_with?("/")
		return true
	elsif target == "*"
		return true
	else
		return false
	end
end

Instance Method Details

#connectionObject



71
72
73
# File 'lib/async/http/protocol/http1/request.rb', line 71

def connection
	@connection
end

#hijack!Object

Hijack the underlying connection for bidirectional communication.



81
82
83
# File 'lib/async/http/protocol/http1/request.rb', line 81

def hijack!
	@connection.hijack!
end

#hijack?Boolean

Returns:

  • (Boolean)


76
77
78
# File 'lib/async/http/protocol/http1/request.rb', line 76

def hijack?
	true
end

#write_interim_response(status, headers = nil) ⇒ Object

Write an interim (1xx) response to the client.



88
89
90
# File 'lib/async/http/protocol/http1/request.rb', line 88

def write_interim_response(status, headers = nil)
	@connection.write_interim_response(@version, status, headers)
end