Class: Async::HTTP::Protocol::HTTP2::Request::Stream

Inherits:
HTTP2::Stream
  • Object
show all
Defined in:
lib/async/http/protocol/http2/request.rb

Overview

Represents the HTTP/2 stream associated with an incoming server-side request.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initializeStream

Initialize the request stream.



18
19
20
21
22
23
# File 'lib/async/http/protocol/http2/request.rb', line 18

def initialize(*)
	super
	
	@enqueued = false
	@request = Request.new(self)
end

Instance Attribute Details

#requestObject (readonly)

Returns the value of attribute request.



25
26
27
# File 'lib/async/http/protocol/http2/request.rb', line 25

def request
  @request
end

Instance Method Details

#closed(error) ⇒ Object

Called when the stream is closed.



116
117
118
119
120
# File 'lib/async/http/protocol/http2/request.rb', line 116

def closed(error)
	@request = nil
	
	super
end

#fail_request(status, error) ⇒ Object

Write a failure response with the given status code and error class name.



30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
# File 'lib/async/http/protocol/http2/request.rb', line 30

def fail_request(status, error)
	body = error.class.name
	headers = [
		[STATUS, status.to_s],
		["content-type", "text/plain; charset=utf-8"],
	]
	
	if @request.head?
		send_headers(headers, ::Protocol::HTTP2::END_STREAM)
	else
		send_headers(headers)
		send_data(body, ::Protocol::HTTP2::END_STREAM)
	end
	
	# The peer may still be sending a request body. The response is complete, so release the stream without reporting a protocol error.
	send_reset_stream(::Protocol::HTTP2::Error::NO_ERROR) unless closed?
rescue => error
	Console.debug(self, "Failed to write failure response!", error)
end

#receive_initial_headers(headers, end_stream) ⇒ Object

Process the initial headers received from the client and construct the request.



53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
# File 'lib/async/http/protocol/http2/request.rb', line 53

def receive_initial_headers(headers, end_stream)
	@headers = ::Protocol::HTTP::Headers.new
	
	headers.each do |key, value|
		if key == SCHEME
			raise ::Protocol::HTTP2::HeaderError, "Request scheme already specified!" if @request.scheme
			
			@request.scheme = value
		elsif key == AUTHORITY
			raise ::Protocol::HTTP2::HeaderError, "Request authority already specified!" if @request.authority
			
			@request.authority = value
		elsif key == METHOD
			raise ::Protocol::HTTP2::HeaderError, "Request method already specified!" if @request.method
			
			@request.method = value
		elsif key == PATH
			raise ::Protocol::HTTP2::HeaderError, "Request path is empty!" if value.empty?
			raise ::Protocol::HTTP2::HeaderError, "Request path already specified!" if @request.path
			
			@request.path = value
		elsif key == PROTOCOL
			raise ::Protocol::HTTP2::HeaderError, "Request protocol already specified!" if @request.protocol
			
			@request.protocol = value
		elsif key == CONTENT_LENGTH
			raise ::Protocol::HTTP2::HeaderError, "Request content length already specified!" if @length
			
			@length = Integer(value)
		elsif key == CONNECTION
			raise ::Protocol::HTTP2::HeaderError, "Connection header is not allowed!"
		elsif key.start_with? ":"
			raise ::Protocol::HTTP2::HeaderError, "Invalid pseudo-header #{key}!"
		elsif key =~ /[A-Z]/
			raise ::Protocol::HTTP2::HeaderError, "Invalid characters in header #{key}!"
		else
			add_header(key, value)
		end
	end
	
	unless @request.valid?
		raise ::Protocol::HTTP2::HeaderError, "Request is missing required headers!"
	else
		# Validate structured headers before exposing the request to the application:
		@headers.to_h
		@request.headers = @headers
		
		# We only construct the input/body if data is coming.
		unless end_stream
			@request.body = prepare_input(@length)
		end
		
		# We are ready for processing:
		@connection.requests.enqueue(@request)
	end
	
	return headers
rescue ::Protocol::HTTP::BadRequest => error
	fail_request(400, error)
end