Class: Async::HTTP::Endpoint

Inherits:
IO::Endpoint::Generic
  • Object
show all
Defined in:
lib/async/http/endpoint.rb

Overview

Represents a way to connect to a remote HTTP server.

Constant Summary collapse

SCHEMES =
{
	"http" => URI::HTTP,
	"https" => URI::HTTPS,
	"ws" => URI::WS,
	"wss" => URI::WSS,
}

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(url, endpoint = nil, **options) ⇒ Endpoint

Returns a new instance of Endpoint.

Parameters:

  • scheme (Hash)

    a customizable set of options

  • hostname (Hash)

    a customizable set of options

  • port (Hash)

    a customizable set of options

  • ssl_context (Hash)

    a customizable set of options

  • alpn_protocols (Hash)

    a customizable set of options

Raises:

  • (ArgumentError)


74
75
76
77
78
79
80
81
82
83
84
85
86
# File 'lib/async/http/endpoint.rb', line 74

def initialize(url, endpoint = nil, **options)
	super(**options)
	
	raise ArgumentError, "URL must be absolute (include scheme, host): #{url}" unless url.absolute?
	
	@url = url
	
	if endpoint
		@endpoint = self.build_endpoint(endpoint)
	else
		@endpoint = nil
	end
end

Instance Attribute Details

#urlObject (readonly)

Returns the value of attribute url.



109
110
111
# File 'lib/async/http/endpoint.rb', line 109

def url
  @url
end

Class Method Details

.[](url) ⇒ Object

Coerce the given object into an endpoint.



61
62
63
64
65
66
67
# File 'lib/async/http/endpoint.rb', line 61

def self.[](url)
	if url.is_a?(Endpoint)
		return url
	else
		Endpoint.parse(url.to_s)
	end
end

.for(scheme, hostname, path = "/", **options) ⇒ Object

Construct an endpoint with a specified scheme, hostname, optional path, and options.



47
48
49
50
51
52
53
54
55
56
57
# File 'lib/async/http/endpoint.rb', line 47

def self.for(scheme, hostname, path = "/", **options)
	# TODO: Consider using URI.for once it becomes available:
	uri_klass = SCHEMES.fetch(scheme.downcase) do
		raise ArgumentError, "Unsupported scheme: #{scheme.inspect}"
	end
	
	self.new(
		uri_klass.new(scheme, nil, hostname, nil, nil, path, nil, nil, nil).normalize,
		**options
	)
end

.parse(string, endpoint = nil, **options) ⇒ Object

Parse a URL string into an endpoint.



36
37
38
39
40
# File 'lib/async/http/endpoint.rb', line 36

def self.parse(string, endpoint = nil, **options)
	url = URI.parse(string).normalize
	
	return self.new(url, endpoint, **options)
end

Instance Method Details

#addressObject



112
113
114
# File 'lib/async/http/endpoint.rb', line 112

def address
	endpoint.address
end

#alpn_protocolsObject



178
179
180
# File 'lib/async/http/endpoint.rb', line 178

def alpn_protocols
	@options.fetch(:alpn_protocols){self.protocol.names}
end

#authority(ignore_default_port = true) ⇒ Object



158
159
160
161
162
163
164
# File 'lib/async/http/endpoint.rb', line 158

def authority(ignore_default_port = true)
	if ignore_default_port and default_port?
		@url.hostname
	else
		"#{@url.hostname}:#{port}"
	end
end

#bind(*arguments, &block) ⇒ Object

Bind to the endpoint.



239
240
241
# File 'lib/async/http/endpoint.rb', line 239

def bind(*arguments, &block)
	endpoint.bind(*arguments, &block)
end

#build_endpoint(endpoint = nil) ⇒ Object

Build a suitable endpoint, optionally wrapping in TLS for secure connections.



212
213
214
215
216
217
218
219
220
221
222
223
224
225
# File 'lib/async/http/endpoint.rb', line 212

def build_endpoint(endpoint = nil)
	endpoint ||= tcp_endpoint
	
	if secure?
		# Wrap it in SSL:
		return ::IO::Endpoint::SSLEndpoint.new(endpoint,
			ssl_context: self.ssl_context,
			hostname: @url.hostname,
			timeout: self.timeout,
		)
	end
	
	return endpoint
end

#connect(&block) ⇒ Object

Connect to the endpoint.



244
245
246
# File 'lib/async/http/endpoint.rb', line 244

def connect(&block)
	endpoint.connect(&block)
end

#default_portObject



133
134
135
# File 'lib/async/http/endpoint.rb', line 133

def default_port
	secure? ? 443 : 80
end

#default_port?Boolean

Returns:

  • (Boolean)


138
139
140
# File 'lib/async/http/endpoint.rb', line 138

def default_port?
	port == default_port
end

#eachObject

Enumerate all resolved endpoints.



251
252
253
254
255
256
257
# File 'lib/async/http/endpoint.rb', line 251

def each
	return to_enum unless block_given?
	
	self.tcp_endpoint.each do |endpoint|
		yield self.class.new(@url, endpoint, **@options)
	end
end

#endpointObject



228
229
230
# File 'lib/async/http/endpoint.rb', line 228

def endpoint
	@endpoint ||= build_endpoint
end

#endpoint=(endpoint) ⇒ Object

Set the underlying endpoint, wrapping it as needed.



234
235
236
# File 'lib/async/http/endpoint.rb', line 234

def endpoint=(endpoint)
	@endpoint = build_endpoint(endpoint)
end

#eql?(other) ⇒ Boolean

Returns:

  • (Boolean)


265
266
267
# File 'lib/async/http/endpoint.rb', line 265

def eql? other
	self.key.eql? other.key
end

#hashObject



270
271
272
# File 'lib/async/http/endpoint.rb', line 270

def hash
	self.key.hash
end

#hostnameObject

The hostname is the server we are connecting to:



148
149
150
# File 'lib/async/http/endpoint.rb', line 148

def hostname
	@options[:hostname] || @url.hostname
end

#inspectObject



105
106
107
# File 'lib/async/http/endpoint.rb', line 105

def inspect
	"\#<#{self.class} #{self.to_url} #{@options.inspect}>"
end

#keyObject



260
261
262
# File 'lib/async/http/endpoint.rb', line 260

def key
	[@url, @options]
end

#localhost?Boolean

Returns:

  • (Boolean)


183
184
185
# File 'lib/async/http/endpoint.rb', line 183

def localhost?
	@url.hostname =~ /^(.*?\.)?localhost\.?$/
end

#pathObject

Return the path and query components of the given URL.



167
168
169
170
171
172
173
174
175
# File 'lib/async/http/endpoint.rb', line 167

def path
	buffer = @url.path || "/"
	
	if query = @url.query
		buffer = "#{buffer}?#{query}"
	end
	
	return buffer
end

#portObject



143
144
145
# File 'lib/async/http/endpoint.rb', line 143

def port
	@options[:port] || @url.port || default_port
end

#protocolObject



122
123
124
125
126
127
128
129
130
# File 'lib/async/http/endpoint.rb', line 122

def protocol
	@options.fetch(:protocol) do
		if secure?
			Protocol::HTTPS
		else
			Protocol::HTTP
		end
	end
end

#schemeObject



153
154
155
# File 'lib/async/http/endpoint.rb', line 153

def scheme
	@options[:scheme] || @url.scheme
end

#secure?Boolean

Returns:

  • (Boolean)


117
118
119
# File 'lib/async/http/endpoint.rb', line 117

def secure?
	["https", "wss"].include?(self.scheme)
end

#ssl_contextObject



197
198
199
200
201
202
203
204
205
206
207
# File 'lib/async/http/endpoint.rb', line 197

def ssl_context
	@options[:ssl_context] || OpenSSL::SSL::SSLContext.new.tap do |context|
		if alpn_protocols = self.alpn_protocols
			context.alpn_protocols = alpn_protocols
		end
		
		context.set_params(
			verify_mode: self.ssl_verify_mode
		)
	end
end

#ssl_verify_modeObject

We don’t try to validate peer certificates when talking to localhost because they would always be self-signed.



188
189
190
191
192
193
194
# File 'lib/async/http/endpoint.rb', line 188

def ssl_verify_mode
	if self.localhost?
		OpenSSL::SSL::VERIFY_NONE
	else
		OpenSSL::SSL::VERIFY_PEER
	end
end

#to_sObject



100
101
102
# File 'lib/async/http/endpoint.rb', line 100

def to_s
	"\#<#{self.class} #{self.to_url} #{@options}>"
end

#to_urlObject



89
90
91
92
93
94
95
96
97
# File 'lib/async/http/endpoint.rb', line 89

def to_url
	url = @url.dup
	
	unless default_port?
		url.port = self.port
	end
	
	return url
end