Module: Async::HTTP::Protocol::Configurable
Overview
Provides a ‘new` method that creates a Configured wrapper, allowing protocols to be instantiated with custom options.
Instance Method Summary collapse
-
#new(**options) ⇒ Object
Create a new Configured instance wrapping this protocol with the given options.
Instance Method Details
#new(**options) ⇒ Object
Create a new Async::HTTP::Protocol::Configured instance wrapping this protocol with the given options.
52 53 54 |
# File 'lib/async/http/protocol/configurable.rb', line 52 def new(**) Configured.new(self, **) end |