Class: Async::HTTP::Protocol::HTTP2::Response::Stream

Inherits:
Stream
  • Object
show all
Defined in:
lib/async/http/protocol/http2/response.rb

Overview

Represents the HTTP/2 stream associated with an outgoing client-side response.

Instance Attribute Summary collapse

Attributes inherited from Stream

#headers, #input, #pool

Instance Method Summary collapse

Methods inherited from Stream

#add_header, #finish_output, #prepare_input, #process_data, #process_headers, #receive_trailing_headers, #send_body, #update_local_window, #window_updated

Constructor Details

#initializeStream

Initialize the response stream.



18
19
20
21
22
23
24
25
# File 'lib/async/http/protocol/http2/response.rb', line 18

def initialize(*)
	super
	
	@response = Response.new(self)
	
	@notification = Async::Notification.new
	@exception = nil
end

Instance Attribute Details

#responseObject (readonly)

Returns the value of attribute response.



27
28
29
# File 'lib/async/http/protocol/http2/response.rb', line 27

def response
  @response
end

Instance Method Details

#accept_push_promise_stream(promised_stream_id, headers) ⇒ Object

Handle a push promise stream from the server.

Raises:

  • (ProtocolError)


42
43
44
# File 'lib/async/http/protocol/http2/response.rb', line 42

def accept_push_promise_stream(promised_stream_id, headers)
	raise ProtocolError, "Cannot accept push promise stream!"
end

#closed(error) ⇒ Object

Called when the stream is closed.



129
130
131
132
133
134
135
136
137
138
139
# File 'lib/async/http/protocol/http2/response.rb', line 129

def closed(error)
	super
	
	if @response
		@response = nil
	end
	
	@exception = error
	
	self.notify!
end

#notify!Object

Notify anyone waiting on the response headers to be received (or failure).



110
111
112
113
114
115
# File 'lib/async/http/protocol/http2/response.rb', line 110

def notify!
	if notification = @notification
		@notification = nil
		notification.signal
	end
end

#receive_initial_headers(headers, end_stream) ⇒ Object

This should be invoked from the background reader, and notifies the task waiting for the headers that we are done.



47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
# File 'lib/async/http/protocol/http2/response.rb', line 47

def receive_initial_headers(headers, end_stream)
	# While in theory, the response pseudo-headers may be extended in the future, currently they only response pseudo-header is :status, so we can assume it is always the first header.
	status_header = headers.shift
	
	if status_header.first != ":status"
		raise ProtocolError, "Invalid response headers: #{headers.inspect}"
	end
	
	status = Integer(status_header.last)
	
	if status >= 100 && status < 200
		return receive_interim_headers(status, headers)
	end
	
	@response.status = status
	@headers = ::Protocol::HTTP::Headers.new
	
	# If the protocol request was successful, ensure the response protocol matches:
	if status == 200 and protocol = @response.request.protocol
		@response.protocol = Array(protocol).first
	end
	
	headers.each do |key, value|
		# It's guaranteed that this should be the first header:
		if key == CONTENT_LENGTH
			@length = Integer(value)
		else
			add_header(key, value)
		end
	end
	
	@response.headers = @headers
	
	if @response.valid?
		if !end_stream
			# We only construct the input/body if data is coming.
			@response.body = prepare_input(@length)
		elsif @response.head?
			@response.body = ::Protocol::HTTP::Body::Head.new(@length)
		end
	else
		send_reset_stream(::Protocol::HTTP2::Error::PROTOCOL_ERROR)
	end
	
	self.notify!
	
	return headers
end

#receive_interim_headers(status, headers) ⇒ Object

Process interim (1xx) response headers.



99
100
101
102
103
104
105
106
107
# File 'lib/async/http/protocol/http2/response.rb', line 99

def receive_interim_headers(status, headers)
	if headers.any?
		headers = ::Protocol::HTTP::Headers[headers]
	else
		headers = nil
	end
	
	@response.request.send_interim_response(status, headers)
end

#waitObject

Wait for the headers to be received or for stream reset.



118
119
120
121
122
123
124
125
# File 'lib/async/http/protocol/http2/response.rb', line 118

def wait
	# If you call wait after the headers were already received, it should return immediately:
	@notification&.wait
	
	if @exception
		raise @exception
	end
end

#wait_for_inputObject

Wait for the response headers and return the response body.



31
32
33
34
35
36
37
# File 'lib/async/http/protocol/http2/response.rb', line 31

def wait_for_input
	# The input isn't ready until the response headers have been received:
	@response.wait
	
	# There is a possible race condition if you try to access @input - it might already be closed and nil.
	return @response.body
end